# -*- Perl -*-
#
# Copyright (C) 1996 Craig Ruefenacht <ruefenac@cs.utah.edu> and the
#                    University of Utah Computer Science Department
#
# $Id: Rust_Data.pm,v 1.33 1997/09/20 19:43:37 ruefenac Exp $
#
# This is part of RUST-1.0 Beta 6 patchlevel 2
#
# Abstract: This library contains a class defination to use to
#           deal with information reqarding a specified ticket
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with the source distribution of Rust, in the license
# directory. If you don't have a copy of it, write to: 
#
# Free Software Foundation, Inc.
# 675 Mass Ave
# Cambridge, MA 02139, USA.

{
package Rust_Data;

use Rust_Configs;
use Rust_Debug;
use Rust_Lib;
use vars qw($globals);

#-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
sub new {
    #
    # Type: Part of package Rust_Data
    #
    # Abstract: Create an empty object to store data about a
    #           specific RUST ticket

    my ($type) = shift;
    my ($self) = {};
    $globals = &Rust_Configs::get_globals;

    bless $self, $type;
}

#-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
sub init {
    #
    # Type: Part of package Rust_Data
    #
    # Abstract: Return the names of the header file and an
    #           error file to work with.
    #
    #           The name of the file which contains the
    #           actual contents of the ticket will be
    #           figured out when the header file is read.
    #
    #           In order to keep the header directories down
    #           to a managable size, it is broken up into
    #           subdirectories, according to the header
    #           file names.  Each header file name is a
    #           number, so I broke it down according to
    #           numerics.  #1-1000 will be someplace,
    #           #1001-2000 will be someplace else,
    #           #2001-3000 will be someplace else, and so on.

    my ($self) = shift;
    my ($num, $index) = @_;

    &ddebug("Entering Rust_Data::init\n");

    my ($mod, $remainder, @new_files, $name);
    my ($path_to_headers) = $globals->{'path_to_headers'};
    my ($path_to_error) = $globals->{'path_to_error'};
    my ($alias_name) = &Rust_Configs::extract_element('alias_name',$index);
    my ($rfc_date) = &Rust_Lib::rfc_822_date;
    my ($date) = &main::ctime($^T);
    chomp $date;
    $date .= " (" . time() . ")";

    # figure out where the header file is, using 1000 as the breakdown
    # reference.  Ticket 1-999 go in "1", 1000-1999 go in "1000",
    # 2000-3999 go in "2000", and so on.  

    $mod = $num % 1000;
    $remainder = $num - $mod;
    if ($remainder == 0) { $remainder = 1; }

    $self->{'header_file'} = "$path_to_headers/$alias_name/$remainder/$num";
    $self->{'error_file'} = "$path_to_error/$alias_name.$num.$$";
    $self->{'alias_name'} = $alias_name;
    $self->{'queue_index'} = $index;
    $self->{'current_date'} = $date;
    $self->{'rfc_date'} = $rfc_date;

    # Need to reset these, because if some of the following do not
    # exist in earlier tickets, and we have already been here
    # reading in another ticket, the values from that other ticket
    # will stay in $self->{}

    $self->{'subject'} = ();
    $self->{'substring'} = ();
    $self->{'notified'} = ();
    $self->{'priority'} = 'normal';
    $self->{'type'} = 'active';
    $self->{'status'} = 'new';
    $self->{'updated'} = ();
    $self->{'created'} = $date;
    $self->{'number'} = ();
    $self->{'eta'} = ();
    $self->{'owner'} = ();
    $self->{'user'} = ();
    $self->{'email'} = ();
    $self->{'other_queues'} = ();
    $self->{'merged'} = 0;
    $self->{'moved'} = 0;
    $self->{'accounting'} = ();
    $self->{'work_hours'} = '0:0';
    $self->{'category'} = ();
    $self->{'access'} = 'public';

    # The header directory may not exist.  Check it, and if it
    # doesn't, then we need to make it.
    
    if (!(-d "$path_to_headers/$alias_name/$remainder")) {
	$mkdir_prog = Rust_Lib::untaint_obj($globals->{'mkdir_prog'});
	$chmod_prog = Rust_Lib::untaint_obj($globals->{'chmod_prog'});
	`$mkdir_prog $path_to_headers/$alias_name/$remainder`;
	`$chmod_prog 2770 $path_to_headers/$alias_name/$remainder`;
    }

    # Now try to read in the ticket information.  If the header
    # file doesn't exist, then we may be dealing with a new ticket,
    # so it isn't necessarily an error.
    
    if (-f $self->{'header_file'}) {

	$merged_ticket = 1;
	$moved_ticket = 1;
	$new_num = $num;
	$new_index = $index;

	while (($merged_ticket) || ($moved_ticket)) {

	    # Go and read the header file of the ticket
	    @errors = Rust_Lib::read_file("","", $self, $num, $index);
	    return $errors[0] if ($errors[0] == 13);
	    
	    $new_self = $errors[1];
	    
	    # Now check to see if this ticket is merged someplace
	    # else...

	    $merged_header = $new_self->get_value('merged');
	    $moved_header = $new_self->get_value('moved');
	    &debug("The moved_header is $moved_header\n");

	    # If the merge header is true, it contains the
	    # ticket number to where the ticket was merged to.
	    # Set things up to read this ticket

	    if ($merged_header) {
		&debug("Found a MERGED ticket\n"); 
		$new_num = (split(',',$merged_header))[0];
		$moved_ticket = 0;

	    # If the move header contains anything besides 0, then
            # the move header should contain "queue_name,ticket_number".		
            # To read the real file, we need to change both the queue_index
	    # and the ticket_number to where the ticket was moved to.
	
	    } elsif ($moved_header) {
	        &debug("Found a MOVED ticket\n");
		($new_queue, $new_num) = (split(',',$moved_header));
		&debug("The new number is $new_num and new queue is $new_queue\n"); 
		$count = 0;
		foreach $each (split(':',$globals->{'alias_name'})) {
		    if ($each eq $new_queue) {
			$new_index = $count;
			$merged_ticket = 0;
		    }
		    $count++;
		}

 	    # If we get here, we have found the real file and
	    # don't have to read anymore.
	    } else {
		$merged_ticket = 0;
		$moved_ticket = 0;
	    }

	    # If the file read is different than the one we thought,
	    # recompute some stuff.

	    if ($merged_ticket || $moved_ticket) {

		$mod = $new_num % 1000;
		$remainder = $new_num - $mod;
		if ($remainder == 0) { $remainder = 1; }
		$num = $new_num;
		
		if ($moved_ticket) {
		    $alias_name = &Rust_Configs::extract_element('alias_name',$new_index);
		    $index = $new_index;
		}

		$self->{'header_file'} = "$path_to_headers/$alias_name/$remainder/$num";
		$self->{'error_file'} = "$path_to_error/$alias_name.$num.$$";
		$self->{'alias_name'} = $alias_name;
		$self->{'queue_index'} = $index;
		$self->{'moved_ticket'} = $moved_ticket;
		$self->{'merged_ticket'} = $merged_ticket;
	    }
	}

        &ddebug("Leaving Rust_Data::init\n");
	return (0, $new_num, $new_index);
    }
    &ddebug("Leaving Rust_Data::init\n");
    return 13;
}


#-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
sub check_for_header_file {
    #
    # Type: Part of package Rust_Data
    #
    # Abstract: Check to see if the header file actually
    #           exists.  If so, return 1, if not, return 0.
    #

    my ($self) = shift;

    if (-f $self->{'header_file'}) { return 1; }
    return 0;
}

#-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
sub read_header_file {
    #
    # Type: Part of package Rust_Data
    #
    # Abstract: Read data in from the header file and store it
    #           in a hash
    #

    my ($self) = shift;
    my (@file_contents); 

    # Read in the header file and store each piece of info in a hash

    open(INPUT,"<$self->{\"header_file\"}");
    @file_contents = <INPUT>;
    close(INPUT);

    foreach $each (@file_contents) {
	chop $each;

	$each =~ /^(.*?)::(.*?)$/;
	$self->{$1} = $2;

	if ($1 eq "number") {
	    $self->{$1} = (split(',',$2))[0];
	}
    }
}

#-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
sub get_value {
    #
    # Type: Part of package Rust_Data
    #
    # Abstract: Looks in the hash for a specified value.  If
    #           it is found, then return the value, if not,
    #           return "unknown".  Can't return something line
    #           0 if it fails, because some values will be
    #           0.

    my ($self) = shift;
    my ($lookup) = @_;
    my (@keys) = keys %$self;
    
    foreach $key (@keys) {
	if($key eq $lookup) { return $self->{$lookup}; }
    }

    return 'unknown';

}

#-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
sub set_value {
    #
    # Type: Part of package Rust_Data
    #
    # Abstract: Set a value in the hash to something
    #

    my ($self) = shift;
    my ($key, $value) = @_;

    $self->{$key} = $value;
}

#-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
sub write_headers {
    #
    # Type: Part of package Rust_Data
    #
    # Abstract: Write the hash out to file.  This is done
    #           when we are done modifying the values in the
    #           hash and we want the contents written out.
    #
    #           This OVERWRITES the old header file. 
    #

    my ($self) = shift;

    my ($header_file) = $self->{'header_file'};
    my (@keys) = keys %$self;

    $header_file = Rust_Lib::untaint_obj($header_file);

    open(OUTPUT,">$header_file");

    # There are a few elements in the hash we don't need
    # in the header file.  These are header_file,
    # error_file, alias_name, and queue_index

    # perl didn't like "$key::$self->{$key}\n" all put together,
    # so it is broken up into seperate lines to fix that.
    
    foreach $key (@keys) {
	if (!(($key eq 'header_file') || ($key eq 'error_file') ||
	      ($key eq 'alias_name') || ($key eq 'queue_index') ||
	      ($key eq 'current_date') || ($keq eq 'merged_ticket') ||
	      ($key eq 'moved_ticket') || ($key eq 'rfc_date'))) {
	    print OUTPUT $key;
	    print OUTPUT "::";
	    print OUTPUT $self->{$key};
	    print OUTPUT "\n";
	}
    }

    close(OUTPUT);
}

#-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
sub create_header_info {
    #
    # Type: Part of package Rust_Data
    #
    # Abstract: Create header information for a new ticket.
    #           This will be written to file later

    my ($self) = shift;
    my ($new_number) = @_;

    my ($date) = &main::ctime($^T);
    chomp $date;
    $date .= " (" . time() . ")";

    $self->{'subject'} = ();
    $self->{'substring'} = ();
    $self->{'notified'} = ();
    $self->{'priority'} = 'normal';
    $self->{'type'} = 'active';
    $self->{'status'} = 'new';
    $self->{'updated'} = ();
    $self->{'created'} = $date;
    $self->{'number'} = $new_number;
    $self->{'eta'} = ();
    $self->{'owner'} = ();
    $self->{'user'} = ();
    $self->{'email'} = ();
    $self->{'other_queues'} = ();
    $self->{'merged'} = 0;
    $self->{'moved'} = 0;
    $self->{'accounting'} = ();
    $self->{'work_hours'} = '0:0';
    $self->{'category'} = ();
    $self->{'access'} = 'public';
}
}

1;

# EOF
